#include <stdlib.h>
#include <complex>
#include <iostream>

using namespace std;


/* рассчёт параметров класса triangle, представляющего треугольник */

class point : public complex<double> {   // класс вершины наследуемый от complex 
private:
   bool bGood;
protected:
   point( void ) : bGood( true ) {
      *(complex<double>*)this = complex<double>( 0.0, 0.0 );
   }
   point( double re, double im ) : bGood( true ) {
      *(complex<double>*)this = complex<double>( re, im );
   }
   point( const complex<double>& c ) : bGood( true ) { 
      *(complex<double>*)this = c;
   }
public:
   friend class triangle;
   inline bool bOK( void ) { return bGood; };
   friend ostream& operator << ( ostream& stream, point& obj );
   friend istream& operator >> ( istream& stream, point& obj );
};

inline ostream& operator << ( ostream& stream, point& obj ) {
   stream << "[" <<  obj.real() << "," << obj.imag() << "]"; 
   return stream;
};

inline istream& operator >> ( istream& stream, point& obj ) {
   double x, y;       
   string s;
   obj.bGood = false;                       // ошибка при неправильном вводе 
   if( ( cin >> x ).eof() ) return stream;  // ввод real
   if( cin.rdstate() & ios::failbit ) {
      cerr << "ошибка ввода!" << endl;
      cin.clear();
      getline( cin, s );
      return stream; 
   }
   if( ( cin >> y ).eof() ) return stream;  // ввод image
   if( cin.rdstate() & ios::failbit ) {
      cerr << "ошибка ввода!" << endl;
      cin.clear();
      getline( cin, s );
      return stream; 
   }
   getline( cin, s );
   if( !s.empty() ) {                       // если введено больше 2-значений
      basic_string<char>::iterator i;
      for( i = s.begin(); i != s.end() && *i == ' '; i++ );
      if( i != s.end() ) {                  // если там непробельные символы
         cerr << "ошибка ввода: " << s << endl;
         return stream; 
      }
   }
   obj = point( complex<double>( x, y ) ); 
   return stream; 
};


class triangle {                         // класс треугольник производный от point
public:
   static const int NODES = 3;           // число вершин
protected:
   point pt[ NODES ];                    // координаты вершин
public:
   double perimeter( void ) {
      double summa = 0.0;
      int i, j;
      for( i = 0; i < NODES; i++ ) {
         j = NODES - 1 == i ? 0 : i + 1;
         summa += abs( pt[ i ] - pt[ j ] );
      } 
      return summa;
   }
   double square( void ) {
      complex<double> side1 = pt[ 1 ] - pt[ 0 ],
                      side2 = pt[ 2 ] - pt[ 0 ];
      return abs( side1 ) * abs( side2 ) * 
             fabs( sin( arg( side1 ) - arg( side2 ) ) ) / 2.; 
   }
   inline point& operator [] ( int i ) { return pt[ i ]; }
   friend istream& operator >> ( istream& stream, triangle& obj ) {
      int i = 0;
      while( i < NODES ) { 
         cout << "вершина № " << i + 1 << " : " << flush;
         stream >> obj.pt[ i ];
         if( stream.eof() ) return stream;
         if( !obj.pt[ i ].bOK() ) continue;
         i++;
      }
      return stream; 
   }
};

int main( int argc, char **argv, char **envp ) {
   int i = 0;
   cout.precision( 3 );
   while( 1 ) {
      triangle polygon;
      cout << "координаты вершин в формате: X Y" << endl;
      if( ( cin >> polygon ).eof() )
         cout << "завершение" << endl, exit( EXIT_SUCCESS );
      cout << "вершин " << triangle::NODES << " : ";
      for( i = 0; i < triangle::NODES; i++ ) 
         cout << polygon[ i ] << " ";
      cout <<  endl << "периметр = " <<  polygon.perimeter() << endl
           << "площадь = " <<  polygon.square() << endl
           << "---------------------------------" << endl; 
   }
}
