import scala.math._  /* символ _ в Scala - "групповой" символ */
import java.util.StringTokenizer

class Complex( re: Double, im: Double ) extends AnyRef {
   private val r: Double = re 
   private val i: Double = im
   def -(that: Complex) =
      new Complex( r - that.r, i - that.i )  
   def real: Double = r 
   def imag: Double = i
   override def toString = "[%.2f,%.2f] ".format( r, i )
   def abs(): Double = sqrt( r * r + i * i )
   def arg(): Double = atan2( i, r )
}

object triangle_sc {

   def perimeter( l: List[ Complex ] ): Double = {
      val h: Complex = l.head
      def distance( p1: Complex, p2: Complex ): Double = ( p1 - p2 ).abs()  
      def sides( l: List[ Complex ] ): Double = {
         if( Nil == l.tail ) distance( l.head, h ) 
         else distance( l.head, l.tail.head ) + sides( l.tail )
      } 
      sides( l )
   }

   def square( l: List[ Complex ] ): Double = {
      val h: Complex = l.head
      def trisq( l: List[ Complex ] ): Double = {
         val s1 = l.head - h
         val s2 = l.tail.head - h
         s1.abs * s2.abs * abs( sin( s1.arg - s2.arg ) ) / 2.0         
      }
      def addsq( l: List[ Complex ] ): Double = {
         if( Nil == l.tail ) 0.0 
         else trisq( l ) + addsq( l.tail )
      } 
      addsq( l.tail )
   }

   def next(): Unit = {
      Console.println( "координаты вершин в формате: X Y (^D конец ввода)" )
      var i: Int = 0
      var eof: Boolean = false
      var shape: List[ Complex ] = List()
      while( !eof ) {
         var s: String = Console.readLine( "%s%d%s", "вершина № ", i + 1, " : " )
         if( null == s ) {
            eof = true              // ^D - конец ввода
            Console.print( "\r" )
         }
         else {                     // в блоке используется Java API:
            try {
               val st: StringTokenizer = new StringTokenizer( s, " \r\n" )
               var x: Double = st.nextElement.toString.toDouble 
               var y: Double = st.nextElement.toString.toDouble  
               try {                // попытка чтения сверх x, y  
                  if( null != st.nextElement.toString )
                     throw new Exception( "Сгенерированное исключения" )
               }
               catch {
                  case ex: NoSuchElementException =>
                     shape = new Complex( x, y ) :: shape
                     i = i + 1
               }
            } 
            catch {                 // все! ошибки ловятся здесь
               case ex: Exception =>
                  Console.println( "ошибка ввода!" )
            }
         }
      }
      Console.print( "вершин " + shape.length + " : " ) 
      shape.map( x => Console.print( x ) )
      Console.println
      Console.println( "периметр = %.2f".format( perimeter( shape ) ) )
      Console.println( "площадь = %.2f".format( square( shape ) ) ) 
      Console.println( "---------------------------------" )
      next()
   }

   def main( args: Array[ String ] ): Unit = next() // главная стартовая подпрограмма

}

