import Glibc

enum InputError: Error {
    case noDigit( s : Substring )
    case emptyString
    case lessDigits
    case moreDigits
}

class point {   // класс 2D точки 
   var x, y : Double
   init( x : Double, y : Double ) {
      self.x = x
      self.y = y
   }
   func display() {
      print( "[\( x ),\( y )]", terminator: " " )
   }
   func minus( p : point ) -> point {
      return point( x:x - p.x, y:y - p.y )
   }
   func abs() -> Double {
      return sqrt( pow( x, 2 ) + pow( y, 2 ) ) 
   }
   func polar() -> ( Double, Double ) {
      let r = abs(), θ = atan2( y, x )
      return ( r, θ )
   }
}

func input() throws -> point? {
   let line : String! = readLine()
   if nil == line { return nil } // EOF
   if 0 == line.count { throw InputError.emptyString }
   let компоненты : Array<Substring> = line.split( whereSeparator: { $0 == " " } )
   if компоненты.count < 2 { throw InputError.lessDigits }
   if компоненты.count > 2 { throw InputError.moreDigits } 
   let x : Double! = Double( компоненты[ 0 ] )
   if x == nil { throw InputError.noDigit( s:компоненты[ 0 ] ) }
   let y : Double! = Double( компоненты[ 1 ] )
   if y == nil { throw InputError.noDigit( s:компоненты[ 1 ] ) }
   return point( x:x, y:y )
} 

class polygon {
   var count = 0
   var pts = [point]()
   func append( p : point ) {
      pts.append( p )
   }
   init() {
      print( "координаты вершин в формате: X Y (^D конец ввода)" )
      while true {
         print( "вершина № \( count + 1 ) : ", terminator: "" )
         do {
            if let p = try input() {       // p = не nil
               pts.append( p )
               count += 1
            }
            else { 
               print( "\r", terminator: "" )
               break                       // ^D - EOF
            }   
         }
         catch InputError.emptyString {
            break;                         // Enter - EOF
         }
         catch InputError.lessDigits {
            print( "мало чисел" ) 
         } 
         catch InputError.moreDigits {
            print( "много чисел" ) 
         }
         catch InputError.noDigit( let why ) {
            print( "не число: \(why)" ) 
         }
         catch {
            print( "непонятная ошибка: \(error)" )
         }
      }
      if 0 == count { return }            
      print( "вершин \(count) : ", terminator: "" )
      for p in pts {
         p.display()
      }
      print()
   }
   func perimeter() -> Double {
      var summa = 0.0
      for i in 1 ... count {
         summa += ( pts[ i % count ].minus( p:pts[ i - 1 ] ) ).abs()
      } 
      return summa
   }
   func square() -> Double {
      var summa = 0.0
      for i in 1 ... count - 2 {
         let ( r1, θ1 ) = ( pts[ i ].minus( p:pts[ 0 ] ) ).polar(),
             ( r2, θ2 ) = ( pts[ i + 1 ].minus( p:pts[ 0 ] ) ).polar()
         summa += r1 * r2 * fabs( sin( θ2 - θ1 ) ) / 2.0 
      }
      return summa
   }
}  

while true {
   let  plg = polygon()
   if( plg.count < 3 ) { break }
   print( "периметр = \( plg.perimeter() )" )
   print( "площадь = \( plg.square() )" )
}
